@@ -7,7 +7,7 @@
 
 #[derive(Copy, Clone)]
 pub struct Argument {
-    pub position: Position,
+    pub position: usize,
     pub format: FormatSpec,
 }
 
@@ -37,12 +37,5 @@ pub enum Alignment {
 pub enum Count {
     Is(usize),
     Param(usize),
-    NextParam,
     Implied,
 }
-
-#[derive(Copy, Clone)]
-pub enum Position {
-    Next,
-    At(usize),
-}
